@@ -26,7 +26,6 @@
     SOFTWARE.
 """
 
-
 __all__ = ['DynamoDbClient', 'clean_dynamo_table']
 __author__ = "Nikolay Grishchenko, Sophie Fogel, Gil Halperin"
 __version__ = "1.6"
@@ -75,6 +74,8 @@ class DynamoDbClient:
         }
 
     """
+
+
     def __init__(self, config):
         assert isinstance(config, dict), "Config must be provided during DynamoDbClient initialization"
 
@@ -104,7 +105,10 @@ class DynamoDbClient:
 
 
     def identify_dynamo_capacity(self, table_name=None):
-        """Identify and store the table capacity for a given table on the object
+        """
+        Identify and store the table capacity for a given table on the object.
+
+        In case the table has ON DEMAND (PAY_PER_REQUEST) BillingMode the ProvisionedThroughput is missing.
 
         Arguments:
             table_name {str} -- short name of the dynamo db table to analyze
@@ -112,19 +116,22 @@ class DynamoDbClient:
         # Use the config value if not provided
         if table_name is None:
             table_name = self.config['table_name']
-            logging.debug("Got `table_name` from config: {table_name}")
+            logging.debug("Got `table_name` from config: %s", table_name)
 
-        logging.debug(f"DynamoDB table name identified as {table_name}")
+        logging.debug("DynamoDB table name identified as %s", table_name)
 
         # Fetch the actual configuration of the dynamodb table directly for
         table_description = self._describe_table(table_name)
-        # Hash to the capacity
-        table_capacity = table_description["Table"]["ProvisionedThroughput"]
 
-        self._table_capacity[table_name] = {
-            'read': int(table_capacity["ReadCapacityUnits"]),
-            'write': int(table_capacity["WriteCapacityUnits"]),
-        }
+        # Hash to the capacity
+        try:
+            table_capacity = table_description["Table"]["ProvisionedThroughput"]
+            self._table_capacity[table_name] = {
+                'read':  int(table_capacity["ReadCapacityUnits"]),
+                'write': int(table_capacity["WriteCapacityUnits"]),
+            }
+        except KeyError:
+            pass
 
 
     def _describe_table(self, table_name: Optional[str] = None) -> Dict:
@@ -138,11 +145,15 @@ class DynamoDbClient:
         table_name = self._get_validate_table_name(table_name)
 
         if self._table_descriptions and table_name in self._table_descriptions:
+            logger.debug("Description taken from cache for table %s: %s ", table_name,
+                         self._table_descriptions[table_name])
             return self._table_descriptions[table_name]
         else:
             table_description = self.dynamo_client.describe_table(TableName=table_name)
             self._table_descriptions[table_name] = table_description
-            return table_description
+            logger.debug("Description for table %s received from API and cached: %s ", table_name,
+                         self._table_descriptions[table_name])
+            return self._table_descriptions[table_name]
 
 
     def get_table_keys(self, table_name: Optional[str] = None) -> Tuple[str, Optional[str]]:
@@ -185,6 +196,9 @@ class DynamoDbClient:
                'index_2_name': ...
            }
 
+        ..  note::
+
+            In case the table has ON DEMAND (PAY_PER_REQUEST) BillingMode the provisioned_throughput is missing.
         """
 
         indexes = {}
@@ -211,23 +225,26 @@ class DynamoDbClient:
                 elif key['KeyType'] == 'RANGE':
                     range_key = key['AttributeName']
 
-            # Get write & read capacity.
-            # global sec. indexes have their own capacities, while a local sec. index uses the capacity of the table.
-            write_capacity = index.get('ProvisionedThroughput', {}).get('WriteCapacityUnits') or \
-                             table_description['ProvisionedThroughput']['WriteCapacityUnits']
-            read_capacity = index.get('ProvisionedThroughput', {}).get('ReadCapacityUnits') or \
-                            table_description['ProvisionedThroughput']['ReadCapacityUnits']
-
             indexes[name] = {
                 'projection_type': projection_type,
-                'hash_key': hash_key,
-                'range_key': range_key,
-                'provisioned_throughput': {
-                    'write_capacity': write_capacity,
-                    'read_capacity': read_capacity
-                }
+                'hash_key':        hash_key,
             }
 
+            if range_key:
+                indexes[name]['range_key'] = range_key
+
+            # Get write & read capacity.
+            # global sec. indexes have their own capacities, while a local sec. index uses the capacity of the table.
+            if index.get('ProvisionedThroughput') or table_description.get('ProvisionedThroughput'):
+                write_capacity = index.get('ProvisionedThroughput', {}).get('WriteCapacityUnits') or \
+                                 table_description['ProvisionedThroughput']['WriteCapacityUnits']
+                read_capacity = index.get('ProvisionedThroughput', {}).get('ReadCapacityUnits') or \
+                                table_description['ProvisionedThroughput']['ReadCapacityUnits']
+                indexes[name]['provisioned_throughput'] = {
+                    'write_capacity': write_capacity,
+                    'read_capacity':  read_capacity
+                }
+
         return indexes
 
 
@@ -261,6 +278,11 @@ class DynamoDbClient:
                 if val_dict:
                     val = val_dict.get(key_type)  # Ex: 1234 or "myvalue"
 
+                    if val is None and key_type not in val_dict:
+                        real_type = list(val_dict.keys())[0]
+                        raise ValueError(f"'{key}' is expected to be of type '{key_type}' in row_mapper, "
+                                         f"but real value is of type '{real_type}'")
+
                     # type_deserializer.deserialize() parses 'N' to `Decimal` type but it cant be parsed to a datetime
                     # so we cast it to either an integer or a float.
                     if key_type == 'N':
@@ -465,7 +487,8 @@ class DynamoDbClient:
 
         cond_expr = " AND ".join(cond_expr_parts)
 
-        select = ('ALL_ATTRIBUTES' if index_name is None else 'ALL_PROJECTED_ATTRIBUTES') if not return_count else 'COUNT'
+        select = (
+            'ALL_ATTRIBUTES' if index_name is None else 'ALL_PROJECTED_ATTRIBUTES') if not return_count else 'COUNT'
 
         logger.debug(cond_expr, filter_values)
         query_args = {
@@ -553,8 +576,10 @@ class DynamoDbClient:
                 f"Unsupported expression for Filtering: {expression}"
             key = words[0]
             result_expr = f"{key} between :st_between_{key} and :en_between_{key}"
-            result_values = self.dict_to_dynamo({f"st_between_{key}": words[2],
-                                                 f"en_between_{key}": words[4]}, add_prefix=':', strict=False)
+            result_values = self.dict_to_dynamo({
+                                                    f"st_between_{key}": words[2],
+                                                    f"en_between_{key}": words[4]
+                                                }, add_prefix=':', strict=False)
         else:
             raise ValueError(f"Unsupported expression for Filtering: {expression}")
 
@@ -699,11 +724,13 @@ class DynamoDbClient:
         # Convert given keys to dynamo syntax
         query_keys = [self.dict_to_dynamo(item) for item in keys_list]
 
+
         # Check if we skipped something - if we did, try again.
         def get_unprocessed_keys(db_result):
             return 'UnprocessedKeys' in db_result and db_result['UnprocessedKeys'] \
                    and table_name in db_result['UnprocessedKeys'] and db_result['UnprocessedKeys'][table_name]['Keys']
 
+
         all_items = []
 
         for query_keys_chunk in chunks(query_keys, 100):
@@ -772,17 +799,26 @@ class DynamoDbClient:
         return query
 
 
-    def put(self, row, table_name=None, overwrite_existing=True):
+    def put(self, row: Dict, table_name: str = None, overwrite_existing: bool = True):
         """
-        Adds a row to the database
+        Writes the row to the DynamoDB table.
 
-        :param dict row:                The row to add to the table. key is column name, value is value.
-        :param string table_name:       Name of the dynamo table to add the row to.
-        :param bool overwrite_existing: Overwrite the existing row if True, otherwise will raise an exception if exists.
+        :param row:                 The row to add to the table. key is column name, value is value.
+        :param table_name:          Name of the dynamo table to add the row to.
+        :param overwrite_existing:  Overwrite the existing row if True, otherwise will raise an exception if exists.
+
+        ..  warning::
+
+            ``overwrite_existing`` option requires the config to have a 'hash_key' parameter with a name of a field.
         """
 
         table_name = self._get_validate_table_name(table_name)
 
+        if not overwrite_existing:
+            assert 'hash_key' in self.config, \
+                "Missing 'hash_key' parameter in the dynamo_db config. You must provide it in order to use the " \
+                "'create' method, or 'overwrite_existing=False' option."
+
         put_query = self.build_put_query(row, table_name, overwrite_existing)
         logger.debug(f"Put to DB: {put_query}")
 
@@ -793,7 +829,16 @@ class DynamoDbClient:
         self.stats['dynamo_put_queries'] += 1
 
 
-    def create(self, row, table_name=None):
+    def create(self, row: Dict, table_name: str = None):
+        """
+        Uses the mechanism of the ``put`` method, but first validates that the item with same hash & [range] key[s]
+        does not exist in the table. Otherwise raises: ``ConditionalCheckFailedException``
+
+        ..  warning:: This method requires the config to have a 'hash_key' parameter with a name of a field.
+
+        :param row:
+        :param table_name:
+        """
         self.put(row, table_name, overwrite_existing=False)
 
 
@@ -859,9 +904,9 @@ class DynamoDbClient:
             update_expr_parts.append(remove_expression)
 
         update_item_query = {
-            'Key':                       keys,  # Ex. {'key_name':   'key_value', ...}
-            'TableName':                 table_name,
-            'UpdateExpression':          " ".join(update_expr_parts)  # Ex. "SET #attr_name = :attr_name ..."
+            'Key':              keys,  # Ex. {'key_name':   'key_value', ...}
+            'TableName':        table_name,
+            'UpdateExpression': " ".join(update_expr_parts)  # Ex. "SET #attr_name = :attr_name ..."
         }
 
         if expression_attributes:
@@ -971,6 +1016,7 @@ class DynamoDbClient:
         """
         return self.stats
 
+
     def get_capacity(self, table_name=None):
         """Fetches capacity for data tables
 
@@ -978,31 +1024,40 @@ class DynamoDbClient:
             table_name {str} -- DynamoDB (default: {None})
 
         Returns:
-            dict -- read/write capacity for the table requested
+            dict -- read/write capacity for the table requested or None for ON_DEMAND (PAY_PER_REQUEST) tables
         """
 
         if table_name is None:
             logging.debug(self.config)
             table_name = self.config['table_name']
 
-        logging.debug(f"DynamoDB table name identified as {table_name}")
+        if table_name not in self._table_descriptions:
+            logger.debug("Getting description from API because not cached yet: %s", table_name)
+            self.identify_dynamo_capacity(table_name=table_name)
 
         if table_name in self._table_capacity.keys():
-            return self._table_capacity[table_name]
-        else:
-            self.identify_dynamo_capacity(table_name=table_name)
+            logger.debug("Found capacity in the cache for table %s: %s", table_name, self._table_capacity[table_name])
             return self._table_capacity[table_name]
 
 
-    def sleep_db(self, last_action_time: datetime.datetime, action: str):
+    def sleep_db(self, last_action_time: datetime.datetime, action: str, table_name=None):
         """
         Sleeps between calls to dynamodb (if it needs to).
         Uses the table's capacity to decide how long it needs to sleep.
+        No need to sleep for ON DEMAND (PAY_PER_REQUEST) tables.
 
         :param last_action_time: Last time when we did this action (read/write) to this dynamo table
         :param action: "read" or "write"
         """
 
+        if table_name is None:
+            logging.debug(self.config)
+            table_name = self.config['table_name']
+
+        # No need to sleep for ON DEMAND (PAY_PER_REQUEST) tables.
+        if not self.get_capacity(table_name=table_name):
+            return
+
         capacity = self.get_capacity()[action]  # Capacity per second
         time_between_actions = 1 / capacity
 
